@@ -179,9 +179,7 @@ inline int glmmr::Covariance::parse(){
   strvec2d re_par_names_;
   intvec3d re_pars_;
   // now process each step of the random effect terms
-  #ifdef R_BUILD
-  if(colnames_.size()!= (unsigned int)data_.cols())Rcpp::stop("colnames length != data columns");
-  #endif 
+  if(colnames_.size()!= data_.cols())throw std::runtime_error("colnames length != data columns");
   
   int nre = form_.re_.size();
   
@@ -206,9 +204,7 @@ inline int glmmr::Covariance::parse(){
         while(getline(check3, intermediate3, ',')){
           auto colidx = std::find(colnames_.begin(),colnames_.end(),intermediate3);
           if(colidx == colnames_.end()){
-            #ifdef R_BUILD
-            Rcpp::stop("variable "+intermediate3+" not in data");
-            #endif 
+            throw std::runtime_error("variable "+intermediate3+" not in data");
           } else {
             int newidx = colidx - colnames_.begin();
             fnvars[iter].push_back(newidx);
@@ -217,9 +213,7 @@ inline int glmmr::Covariance::parse(){
       } else {
         auto colidx = std::find(colnames_.begin(),colnames_.end(),intermediate2);
         if(colidx == colnames_.end()){
-          #ifdef R_BUILD
-          Rcpp::stop("variable "+intermediate2+" not in data");
-          #endif
+          throw std::runtime_error("variable "+intermediate2+" not in data");
         } else {
           int newidx = colidx - colnames_.begin();
           fnvars[iter].push_back(newidx);
@@ -233,15 +227,13 @@ inline int glmmr::Covariance::parse(){
     dblvec2d groups;
     dblvec vals;
     bool isgr;
-    unsigned int j,k,idx,zcol;
+    int j,k,idx,zcol;
     if(form_.z_[i].compare("1")==0){
       zcol = -1;
     } else {
       auto idxz = std::find(colnames_.begin(),colnames_.end(),form_.z_[i]);
       if(idxz == colnames_.end()){
-        #ifdef R_BUILD
-        Rcpp::stop("z variable "+form_.z_[i]+" not in column names");
-        #endif
+        throw std::runtime_error("z variable "+form_.z_[i]+" not in column names");
       } else {
         zcol = idxz - colnames_.begin();
       }
@@ -286,9 +278,9 @@ inline int glmmr::Covariance::parse(){
     re_cols_data_.resize(currresize+groups.size());
     
     int fn_var_counter = 0;
-    for(unsigned int m = 0; m < fnvars.size(); m++){
+    for(int m = 0; m < fnvars.size(); m++){
       intvec iter_fn_var_index;
-      for(unsigned int p = 0; p < fnvars[m].size(); p++){
+      for(int p = 0; p < fnvars[m].size(); p++){
         iter_fn_var_index.push_back(p + fn_var_counter);
       }
       fn_var_counter += fnvars[m].size();
@@ -298,9 +290,7 @@ inline int glmmr::Covariance::parse(){
     for(j = 0; j < groups.size(); j++){
       fn_.emplace_back();
       for(const auto& fnvalue: fn){
-#ifdef R_BUILD
-        if(glmmr::validate_fn(fnvalue))Rcpp::stop("Function " + fnvalue + " not valid");
-#endif
+        if(glmmr::validate_fn(fnvalue))throw std::runtime_error("Function " + fnvalue + " not valid");
         fn_.back().push_back(str_to_covfunc.at(fnvalue));
       }
       z_.push_back(zcol);
@@ -310,7 +300,7 @@ inline int glmmr::Covariance::parse(){
     
     for(k = 0; k < data_.rows(); k++){
       if(isgr){
-        for(unsigned int m = 0; m < vals.size(); m++) vals[m] = data_(k,fnvars[idx][m]);
+        for(int m = 0; m < vals.size(); m++) vals[m] = data_(k,fnvars[idx][m]);
         auto gridx2 = std::find(groups.begin(),groups.end(),vals);
         gridx = gridx2 - groups.begin();
       } else {
@@ -336,7 +326,7 @@ inline int glmmr::Covariance::parse(){
   int remidx;
   for(int i = 0; i < nre; i++){
     firsti = true;
-    for(unsigned int j = 0; j < fn_.size(); j++){
+    for(int j = 0; j < fn_.size(); j++){
       if(re_order_[j]==i){
         if(firsti){
           intvec2d parcount1;
@@ -367,9 +357,9 @@ inline int glmmr::Covariance::parse(){
     }
   }
   
-  for(unsigned int i =0; i<fn_.size();i++){
-    for(unsigned int j = 0; j < re_pars_[i].size(); j++){
-      for(unsigned int k = 0; k < re_pars_[i][j].size(); k++){
+  for(int i =0; i<fn_.size();i++){
+    for(int j = 0; j < re_pars_[i].size(); j++){
+      for(int k = 0; k < re_pars_[i][j].size(); k++){
         calc_[i].parameter_indexes.push_back(re_pars_[i][j][k]);
       }
     }
@@ -377,18 +367,18 @@ inline int glmmr::Covariance::parse(){
   
   //now build the reverse polish notation and add distances
   int nvarfn;
-  for(unsigned int i =0; i<fn_.size();i++){
+  for(int i =0; i<fn_.size();i++){
     std::vector<Do> fn_instruct;
     intvec fn_par;
     int minvalue = 100;
     int ndata = re_temp_data_[i].size();
     calc_[i].data.conservativeResize(ndata*(ndata-1)/2,fn_[i].size());
     calc_[i].data_size = ndata;
-    for(unsigned int j = 0; j<fn_[i].size();j++){
+    for(int j = 0; j<fn_[i].size();j++){
       auto min_value_iterator = std::min_element(re_pars_[i][j].begin(),re_pars_[i][j].end());
       if(*min_value_iterator < minvalue) minvalue = *min_value_iterator;
     }
-    for(unsigned int j = 0; j<fn_[i].size();j++){
+    for(int j = 0; j<fn_[i].size();j++){
       if(fn_[i][j]!=CovFunc::gr){
         nvarfn = re_cols_[i][j].size();
         double dist_val;
@@ -411,13 +401,13 @@ inline int glmmr::Covariance::parse(){
       }
       std::vector<Do> B = glmmr::interpret_re(fn_[i][j]);
       intvec re_par_less_min_ = re_pars_[i][j];
-      for(unsigned int k = 0; k < re_pars_[i][j].size(); k++)re_par_less_min_[k] -= minvalue;
+      for(int k = 0; k < re_pars_[i][j].size(); k++)re_par_less_min_[k] -= minvalue;
       intvec Bpar = glmmr::interpret_re_par(fn_[i][j],j,re_par_less_min_);
       fn_instruct.insert(fn_instruct.end(),B.begin(),B.end());
       fn_par.insert(fn_par.end(),Bpar.begin(),Bpar.end());
     }
     if(fn_[i].size() > 1){
-      for(unsigned int j = 0; j < (fn_[i].size()-1); j++){
+      for(int j = 0; j < (fn_[i].size()-1); j++){
         fn_instruct.push_back(Do::Multiply);
       }
     }
@@ -431,14 +421,14 @@ inline int glmmr::Covariance::parse(){
   int Qn = 0;
   block_size.resize(calc_.size());
   block_nvar.resize(calc_.size());
-  for(unsigned int i = 0; i < calc_.size(); i++){
+  for(int i = 0; i < calc_.size(); i++){
     Qn += re_temp_data_[i].size();
     block_size[i] = re_temp_data_[i].size();
     block_nvar[i] = re_temp_data_[i][0].size();
   }
   re_count_.resize(form_.re_terms().size());
   std::fill(re_count_.begin(), re_count_.end(), 0);
-  for(unsigned int i = 0; i < calc_.size(); i++) re_count_[re_order_[i]] += re_temp_data_[i].size();
+  for(int i = 0; i < calc_.size(); i++) re_count_[re_order_[i]] += re_temp_data_[i].size();
   B_ = calc_.size();
   n_ = data_.rows();
   return Qn;
@@ -517,9 +507,7 @@ inline int glmmr::Covariance::B() const
 
 inline int glmmr::Covariance::Q() const
 {
-#ifdef R_BUILD
-  if(Q_==0)Rcpp::stop("Random effects not initialised");
-#endif
+  if(Q_==0)throw std::runtime_error("Random effects not initialised");
   return Q_;
 }
 
@@ -563,9 +551,7 @@ inline void glmmr::Covariance::update_parameters(const dblvec& parameters)
 
 inline void glmmr::Covariance::update_parameters_extern(const dblvec& parameters)
 {
-  #ifdef R_BUILD
-  if(parameters.size()!=(unsigned)npar())Rcpp::stop(std::to_string(parameters.size())+" covariance parameters provided, "+std::to_string(npar())+" required");
-  #endif
+  if(parameters.size()!=npar())throw std::runtime_error(std::to_string(parameters.size())+" covariance parameters provided, "+std::to_string(npar())+" required");
   if(parameters_.size()==0){
     parameters_.resize(npar());
   }
@@ -583,20 +569,18 @@ inline void glmmr::Covariance::update_parameters_extern(const dblvec& parameters
 inline void glmmr::Covariance::update_parameters(const ArrayXd& parameters)
 {
   if(parameters_.size()==0){
-    for(unsigned int i = 0; i < parameters.size(); i++){
+    for(int i = 0; i < parameters.size(); i++){
       parameters_.push_back(parameters(i));
     }
     update_parameters_in_calculators();
   } else if(parameters_.size() == parameters.size()){
-    for(unsigned int i = 0; i < parameters.size(); i++){
+    for(int i = 0; i < parameters.size(); i++){
       parameters_[i] = parameters(i);
     }
     update_parameters_in_calculators();
     update_ax();
   } else {
-#ifdef R_BUILD
-    Rcpp::stop(std::to_string(parameters.size())+" covariance parameters provided, "+std::to_string(parameters_.size())+" required");
-#endif
+    throw std::runtime_error(std::to_string(parameters.size())+" covariance parameters provided, "+std::to_string(parameters_.size())+" required");
   }
 };
 
@@ -660,9 +644,7 @@ inline MatrixXd glmmr::Covariance::get_chol_block(int b,bool upper)
 
 inline VectorXd glmmr::Covariance::sim_re()
 {
-  #ifdef R_BUILD
-  if(parameters_.size()==0)Rcpp::stop("no parameters, cannot simulate random effects");
-  #endif
+  if(parameters_.size()==0)throw std::runtime_error("no parameters, cannot simulate random effects");
 #if defined(R_BUILD) && defined(ENABLE_DEBUG)
   Rcpp::Rcout << "\nSim";
 #endif
@@ -762,10 +744,7 @@ inline MatrixXd glmmr::Covariance::Lu(const MatrixXd& u){
 }
 
 inline double glmmr::Covariance::log_likelihood(const VectorXd &u){
-#ifdef R_BUILD
-  if(parameters_.size()==0)Rcpp::stop("no covariance parameters, cannot calculate log likelihood");
-#endif
-  
+  if(parameters_.size()==0)throw std::runtime_error("no covariance parameters, cannot calculate log likelihood");
   double logdet_val=0.0;
   double loglik_val=0.0;
   int obs_counter=0;
@@ -807,9 +786,7 @@ inline double glmmr::Covariance::log_likelihood(const VectorXd &u){
 }
 
 inline double glmmr::Covariance::log_determinant(){
-#ifdef R_BUILD
-  if(parameters_.size()==0)Rcpp::stop("no covariance parameters, cannot calculate log determinant");
-#endif
+  if(parameters_.size()==0)throw std::runtime_error("no covariance parameters, cannot calculate log determinant");
   int blocksize;
   double logdet_val = 0.0;
   if(!isSparse){
@@ -829,9 +806,7 @@ inline double glmmr::Covariance::log_determinant(){
 
 
 inline void glmmr::Covariance::make_sparse(){
-#ifdef R_BUILD
-  if(parameters_.size()==0)Rcpp::stop("no covariance parameters, cannot make sparse");
-#endif
+  if(parameters_.size()==0)throw std::runtime_error("no covariance parameters, cannot make sparse");
   int dim;
   double val;
   int col_counter=0;
@@ -970,8 +945,8 @@ inline MatrixXd glmmr::Covariance::D_sparse_builder(bool chol,
 
 inline bool glmmr::Covariance::any_group_re() const{
   bool gr = false;
-  for(unsigned int i = 0; i < fn_.size(); i++){
-    for(unsigned int j = 0; j < fn_[i].size(); j++){
+  for(int i = 0; i < fn_.size(); i++){
+    for(int j = 0; j < fn_[i].size(); j++){
       if(fn_[i][j]==CovFunc::gr){
         gr = true;
         break;
@@ -984,7 +959,7 @@ inline bool glmmr::Covariance::any_group_re() const{
 
 inline strvec glmmr::Covariance::parameter_names(){
   strvec parnames;
-  for(unsigned int i = 0; i < form_.re_.size(); i++){
+  for(int i = 0; i < form_.re_.size(); i++){
     for(int j = 0; j < B_; j++){
       if(re_order_[j]==i){
         parnames.insert(parnames.end(),calc_[j].parameter_names.begin(),calc_[j].parameter_names.end());
@@ -1105,23 +1080,6 @@ inline VectorXd glmmr::Covariance::log_gradient(const MatrixXd &umat, double& lo
     logl -= 0.5 * qf;
   }
 
-  // VectorXd grad2(npars);
-  // MatrixXd Sinv = D();
-  // Sinv = Sinv.llt().solve(MatrixXd::Identity(Sinv.rows(),Sinv.cols()));
-  // for(int i = 1; i < derivs.size(); i++)
-  //   {
-  //     MatrixXd Sdi = Sinv * derivs[i];
-  //     double dlogdet = Sdi.trace();
-  //     grad2(i-1) = -1.0*dlogdet;
-  //     MatrixXd dinv = Sdi * Sinv;
-  //     double val = 0;
-  //     for(int j = 0; j < niter; j++)
-  //       {
-  //          val += ((umat.col(j).transpose()*dinv)*umat.col(j))(0);
-  //       }
-  //     grad2(i-1) += val/((double)niter);
-  //     grad2(i-1) *= 0.5;
-  //   }
 #if defined(R_BUILD) && defined(ENABLE_DEBUG)
   Rcpp::Rcout << "\nTHETA: ";
   for(const auto& x: parameters_) Rcpp::Rcout << x << " ";
