@@ -22,9 +22,8 @@ public:
   linpred               linear_predictor;
   glmmr::ModelExtraData data;
   glmmr::Family         family;
-  glmmr::calculator     calc;
-  glmmr::calculator     vcalc;
   bool                  weighted = false;
+  int                   trace = 0;
     
   ModelBits(const std::string& formula_,const ArrayXXd& data_,const strvec& colnames_,std::string family_,std::string link_);
   //functions
@@ -32,8 +31,6 @@ public:
   virtual ArrayXd   xb() {return linear_predictor.xb().array() + data.offset.array();};
   virtual void      make_covariance_sparse(bool amd = true);
   virtual void      make_covariance_dense();
-private:
-  void              setup_calculator();
 };
 
 }
@@ -48,34 +45,7 @@ inline glmmr::ModelBits<cov, linpred>::ModelBits(const std::string& formula_,
   covariance(formula,data_,colnames_),
   linear_predictor(formula,data_,colnames_),
   data(data_.rows()),
-  family(family_,link_) { setup_calculator(); };
-
-template<typename cov, typename linpred>
-inline void glmmr::ModelBits<cov, linpred>::setup_calculator(){
-  dblvec yvec(n(),0.0);
-  calc = linear_predictor.calc;
-  glmmr::linear_predictor_to_link(calc,family.link);
-  glmmr::link_to_likelihood(calc,family.family);
-  calc.y = yvec;
-  calc.variance.conservativeResize(yvec.size());
-  calc.variance = data.variance;
-  vcalc = linear_predictor.calc;
-  glmmr::re_linear_predictor(vcalc,covariance.Q());
-  glmmr::linear_predictor_to_link(vcalc,family.link);
-  glmmr::link_to_likelihood(vcalc,family.family);
-  vcalc.y = yvec;
-  vcalc.variance.conservativeResize(yvec.size());
-  vcalc.variance = data.variance;
-  vcalc.data.conservativeResize(NoChange,covariance.Q());
-  vcalc.parameters.resize(covariance.Q());
-  std::fill(vcalc.parameters.begin(),vcalc.parameters.end(),0.0);
-}
-
-template<>
-inline void glmmr::ModelBits<glmmr::hsgpCovariance, glmmr::LinearPredictor>::setup_calculator(){
-  int i = 0;
-  (void)i;
-}
+  family(family_,link_) {};
 
 template<typename cov, typename linpred>
 inline void glmmr::ModelBits<cov, linpred>::make_covariance_sparse(bool amd){
